"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ContainerException_1 = require("./Exceptions/ContainerException");
const ExistsException_1 = require("./Exceptions/ExistsException");
const Factory_1 = require("./Factory");
const NotFoundException_1 = require("./Exceptions/NotFoundException");
class Container {
    constructor(factories = new Map()) {
        this.storedFactories = new Map();
        this.storedPrototypeFactories = new Map();
        this.storedServices = new Map();
        this.factories(factories);
    }
    factories(factories) {
        factories.forEach((factory, id) => {
            this.factory(id, factory);
        });
        return this;
    }
    factory(id, factory) {
        if (this.storedPrototypeFactories.has(id)) {
            throw ExistsException_1.default.create(id, ExistsException_1.default.TYPE_PROTOTYPE_FACTORY);
        }
        if (this.storedFactories.has(id)) {
            factory = Factory_1.default(this.storedFactories.get(id), factory);
        }
        this.storedServices.delete(id);
        this.storedFactories.set(id, factory);
        return this;
    }
    prototypeFactories(factories) {
        factories.forEach((factory, id) => {
            this.prototypeFactory(id, factory);
        });
        return this;
    }
    prototypeFactory(id, factory) {
        if (this.storedFactories.has(id)) {
            throw ExistsException_1.default.create(id, ExistsException_1.default.TYPE_FACTORY);
        }
        if (this.storedPrototypeFactories.has(id)) {
            factory = Factory_1.default(this.storedPrototypeFactories.get(id), factory);
        }
        this.storedPrototypeFactories.set(id, factory);
        return this;
    }
    get(id) {
        const prototypeFactory = this.storedPrototypeFactories.get(id);
        if (prototypeFactory) {
            return this.createFromPrototypeFactory(id, prototypeFactory);
        }
        if (!this.storedServices.has(id)) {
            this.storedServices.set(id, this.create(id));
        }
        return this.storedServices.get(id);
    }
    has(id) {
        return this.storedFactories.has(id) || this.storedPrototypeFactories.has(id);
    }
    create(id) {
        const factory = this.storedFactories.get(id);
        if (!factory) {
            throw NotFoundException_1.default.create(id);
        }
        try {
            return factory(this);
        }
        catch (e) {
            throw ContainerException_1.default.create(id, e);
        }
    }
    createFromPrototypeFactory(id, prototypeFactory) {
        try {
            return prototypeFactory(this);
        }
        catch (e) {
            throw ContainerException_1.default.create(id, e);
        }
    }
}
exports.default = Container;
